/*******************************************************************************
 *
 * Filename : tripapi_types.h
 * Copyright: Smaser AG
 * Purpose  : Type definitions for the TRIP API
 *
 */

#ifndef __TRIPAPI_TYPES_H__
#define __TRIPAPI_TYPES_H__

#include "tripapi_defs.h"

#if !defined(__RPCNDR_H__)
    /* These data types are defined rpcndr.h */
    typedef unsigned char boolean;
    typedef unsigned char byte;
#endif

typedef void* tdb_handle;
typedef void* TdbHandle;
typedef unsigned char Char;


/**
 * Option for function TdbGetFocus2\n
 * \tFOCUS - show hits. in case no hits that return empty (zero length) string \n
 * \tEXTRACT - extract the first n number of words. Do not care about hits. \n
 * \tFOCUS_OR_EXTRACT - do FOCUS, if there is not hits then do EXTRACT
 */
typedef enum GetFocus2Options
{
    FOCUS,
    EXTRACT,
    FOCUS_OR_EXTRACT
} GetFocus2Options;

typedef enum
{
    TRIPVARIANT_NONE = 0,
    TRIPVARIANT_INT = 1,
    TRIPVARIANT_STR = 2
}
variant_type;

typedef struct trip_variant
{
    variant_type type;
    union
    {
        int i32;
        char* str;
    } v;
}
trip_variant;


/* Shell/DAO interface record -- new programs should not use this data type
 * but rather use TdbStartTrip() / TdbStopTrip() / TdbExecuteCcl().
 */
typedef struct
{
    char    shin[400],      /* CCL command input */
            shout[400],     /* Parsed CCL output */
            dao[400];       /* DAO equivalent of shout */
    int     shinlen,   /* Length of CCL command input */
            shoutlen,  /* Length of parsed CCL output */
            daolen,    /* Length DAO equivalent */
            shdind;    /* Shell/DAO indicator */
} shell_dao_rec, ShellDaoRecord;


/* Structure definition for a database */
typedef struct
{
    char    baf_file[132],      /* Base file name */
            bif_file[132],      /* Primary index filename */
            vif_file[132],      /* Secondary index filename */
            log_file[132],      /* Optional transaction log filename */
            word_chars[132],    /* Any "special" searchable characters */
            baffit_ase1[32],    /* ASE called by Baffit before storing record */
            baffit_ase2[32],    /* ASE called by Baffit after storing record */
            container[16],      /* Name of classification container used by DB */
            _pad1[6];
    boolean graph;              /* Is this a graph database? */
    boolean xml;                /* Is this an XML-enabled database? */
    char    chi_words;          /* Chinese word segmentation option (0,1,2) */
    boolean auto_reorg,         /* Does this database allow automatic reorganization of indexes? */
            audit,              /* Does this database have an audit trail? */
            log_delete;         /* Should deletes be written to the transaction log? */
    char    pid_field[16],      /* Name of the part name field */
            char_set[4],        /* Character set used by the database */
            language[3];        /* Language used by the database */
    char    _pad2,
            appl_id[16],        /* Application-specific ID information */
            output_format[16],  /* Name of default output format */
            entry_form[16],     /* Name of default TRIPclassic entry form */
            rid_field[16],      /* Name of the record name field */
            counter_field[16],  /* Name of the record number field */
            char_folding[3];    /* Character folding class */
    char    encrypted;          /* Encryption state mask (reserved for future use) */
    char    base_desc[256],     /* Description of the database */
            submit_que[256];    /* DEPRECATED. Queue used for batch job submissions */
    int     submit_opt;         /* DEPRECATED. Batch job submission options */
    char    submit_ase1[32],    /* DEPRECATED. ASE called before batch job submission */
            submit_ase2[32];    /* DEPRECATED. ASE called after batch job submission */
} base_spec_rec, BaseSpecRecord;


/* Structure used to define custom sentence / paragraph boundary markers */
typedef struct
{
    boolean sent_check,         /* Are sentence numbers to be generated? */
            para_check,         /* Are paragraph numbers to be generated? */
            sent_end_req,       /* Are sentence end markers required for end of paragraph? */
            sent_beg_req;       /* Are sentence begin markers required for begin of paragraph? */
    char    sent_beg_spec[8],   /* Sentence begin specification: UB */
            sent_sep_spec[8],   /* Sentence end specification: SN */
            para_sep_spec[8],   /* Paragraph end specification: 2N */
            sent_end_chars[16], /* Sentence end characters, class E */
            spec_beg_chars[32], /* Sentence begin characters, class B */
            ignore_chars[64];   /* Ignore characters, class I */
} text_sepa_rec, TextSepaRecord;


/* Structure used to define a field's design properties */
typedef struct
{
    char    field_name[16],     /* Name of the field */
            field_type[16],     /* Type of the field */
            copyright_field[16];/* Name of the copyright holder field */
    int     index;              /* Index mode for field */
    boolean orig,               /* Original layout? */
            rid_field,          /* Record name field? */
            pattern,            /* Is there a pattern-based restriction? */
            file_ref,           /* Is there a database-based restriction? */
            pid_field,          /* Part name field? */
            default_value,      /* Is there a default value for this field? */
            unique,             /* Field values must be unique */
            concordance,        /* Used for classification */
            counter_field,      /* Record number field? */
            part_field;         /* Part field? */
    int     field_number,       /* Field number */
            cost,               /* Cost of viewing the field */
            min_items,          /* Minimum number of allowable subfields / paragraphs */
            max_items;          /* Maximum number of allowable subfields / paragraphs */
    char    comment[256],       /* Comment, description */
            restrictions[256],  /* Any restrictions, as specified by flags above */
            baffit_ase[32],     /* ASE called by Baffit when loading field */
            scanit_ase[32];     /* ASE called by Sciffit when indexing field */
    int     error_at;           /* Error flag */
} field_spec_rec, FieldSpecRecord;


/******************************************************************************
 *                                                                            *
 * Each element of the database access record ( except the scope fields ) are *
 * made up using a bit mask as follows :                                      *
 *                                                                            *
 *    bit 0    read access                                                    *
 *    bit 1    write access                                                   *
 *                                                                            *
 * Thus, to give full read only access, set "all_access" to 1. To give full   *
 * read and write access, set "all_access" to 3. If any bits in "sel_access"  *
 * are set, the "field_access" array is read, each element of the array       *
 * corresponding to the access rights to a particular field in the database.  *
 *                                                                            *
 ******************************************************************************/
typedef struct
{
    char    all_access,         /* Unrestricted access to all fields */
            sel_access,         /* Restrict access to certain fields */
            field_access[1000], /* Fields to be given access */
            read_scope[400],    /* CCL command defining read scope */
            write_scope[400];   /* CCL command defining write scope */
} base_access_rec, BaseAccessRecord;


/* Structure used for defining clusters of databases */
typedef struct
{
    int     db_count;                   /* Number of databases in the cluster */
    char    db_names[30][16],           /* Names of databases in the cluster */
            cluster_desc[256];          /* Comment, description */
} cluster_spec_rec, ClusterSpecRecord;


/* Structure used for defining user properties */
typedef struct
{
    char    user_name[60],          /* TRIP username */
            company[60],            /* Company name */
            address1[60],           /* Address */
            address2[60],
            phone[20],
            start_record[33],       /* Procedure automatically run upon login */
            dateform_sep[2];        /* Default date separators */
    int     dateform_nr,            /* ID of default date format */
            start_module;           /* Start module -- TRIPclassic only */
    boolean user_manager,           /* Is user a UM? */
            file_manager,           /* Is used a FM? */
            os_entry;               /* Bypass password checks? */
    int     group_count,            /* Nr of groups to which user belongs */
            group_ids[100];         /* CONTROL record IDs for group records */
} user_profile_rec, UserProfileRecord;


/* Structure used to define hit points in TdbGetFieldInfo() */
typedef struct
{
    int     iLineNr,                /* Line number of field on which hit occurs */
            iHitStart,              /* Column offset in line */
            iHitLength;             /* Length of hit */
} hit_point, HitPoint, *TdbHitVector;


/* Obsolete data structure definitions */
typedef int qbe_vector_entry, QbeVectorEntry, *TdbQbeVector;
typedef int tom_spec_rec;


/* Structure used with TdbImport and TdbExport                      */
typedef struct
{
   TdbHandle    record_control;
   TdbHandle    cursor;
   TdbHandle    filter_address;
   char         filter_name[32];
   char         filter_lib_env[32];
   void*        buffer;
   int          buffer_length;
   char*        filter_arguments;
   int          arg_length;
   int          filter_options;
   int          tdb_options;
   int          blockno;
   int          errorcode;
   char         errortext[256];
} filter_data, FilterData;

typedef struct _tag_integrity_rule
{
    int             type;                   /* 0 for source of link, 1 for target of link */
    int             on_delete;
    int             on_update;
    char*   key_field;
    char*   link_database;
    char*   link_field;
} integrity_rule, IntegrityRule;

typedef struct _tag_base_integrity_spec
{
    int                             nr;
    integrity_rule  rules[1];
} base_integrity_spec, BaseIntegritySpec;

/* Interface structures for classification */
typedef struct _tag_subsystem_info
{
    int             size;
    char*   name;
    boolean requires_finalize;
    boolean supports_hierarchy;
} subsystem_info, SubsystemInfo;

/**
 * Software build version structure.
 * DEPRECATED: Use TdbGetBuildInfo instead!
 */
typedef struct tServerInfo {
    int struct_version; ///< identifies version (=content) of this structure
    int version;
    int major;
    int minor;
    int patch;
    int buildno;
    int server_pid; ///< runtime this process ID
} tServerInfo;

#endif

